/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2013.02.02.
*   Chip         -  Atmel ATmega8
*   Compiler     -  avr-gcc (WinAVR 20080610) 4.3.0
*
*   Definitions for ATMega8
*
********************************************************************************
*   PonyProg Configuration and Security Bits (bepipalva):
*
*   CKSEL3, CKSEL2, CKSEL1, CKSEL0
*   P       P       P       P       0000  External Clock
*   
*   P       P       P       -       0001  Calibrated Internal RC Oscillator 1MHz
*   P       P       -       P       0010  Calibrated Internal RC Oscillator 2MHz
*   P       P       -       -       0011  Calibrated Internal RC Oscillator 4MHz
*   P       -       P       P       0100  Calibrated Internal RC Oscillator 8MHz
*
*   P       -       P       -       0101  External RC Oscillator 0.1-0.9MHz
*   P       -       -       P       0110  External RC Oscillator 0.9-3MHz
*   P       -       -       -       0111  External RC Oscillator 3-8MHz
*   -       P       P       P       1000  External RC Oscillator 8MHz-
*
*   -       P       P       -       1001  External Low-frequency Crystal
*
*   -       P       -       P       1010  Ceramic resonator 0.4-0.9MHz
*   -       P       -       -       1011  Crystal Oscillator 0.4-0.9MHz
*   -       -       P       P       1100  Ceramic resonator 0.9-3MHz
*   -       -       P       -       1101  Crystal Oscillator 0.9-3MHz
*   -       -       -       P       1110  Ceramic resonator 3-8MHz
*   -       -       -       P       1110  Ceramic resonator 8MHz-, +CKOPT P
*   -       -       -       -       1111  Crystal Oscillator 3-8MHz
*   -       -       -       -       1111  Crystal Oscillator 8MHz-, +CKOPT P
*
*   Calibrated Internal RC Oscillator
*   SUT1  SUT0
*   P     P         BOD enabled
*   P     -         Fast rising power
*   -     P         Slowly rising power
*
*******************************************************************************/

#include <avr/sfr_defs.h>


#define TWBR    _SFR_IO8( 0x00)        // TWI stands for "Two Wire Interface" or "TWI Was I2C(tm)"

#define TWSR    _SFR_IO8( 0x01)
#define TWS7    7
#define TWS6    6
#define TWS5    5
#define TWS4    4
#define TWS3    3
// bit 2 reserved
#define TWPS1   1
#define TWPS0   0

#define TWAR    _SFR_IO8( 0x02)
#define TWA6    7
#define TWA5    6
#define TWA4    5
#define TWA3    4
#define TWA2    3
#define TWA1    2
#define TWA0    1
#define TWGCE	  0

#define TWDR    _SFR_IO8( 0x03)

#define ADCW    _SFR_IO16( 0x04)       // ADC Data Register
#define ADCL    _SFR_IO8( 0x04)        // ADC Data Register L
#define ADCH    _SFR_IO8( 0x05)        // ADC Data Register H

#define ADCSR   _SFR_IO8( 0x06)
#define ADCSRA  _SFR_IO8( 0x06)        // ADC Control and Status Register A
#define ADEN    7
#define ADSC    6
#define ADFR    5
#define ADIF    4
#define ADIE    3
#define ADPS2   2
#define ADPS1   1
#define ADPS0   0

#define ADMUX   _SFR_IO8(0x07)         // ADC Multiplexer Selection Register
#define REFS1   7
#define REFS0   6
#define ADLAR   5
// bit 4 reserved
#define MUX3    3
#define MUX2    2
#define MUX1    1
#define MUX0    0

#define ACSR    _SFR_IO8( 0x08)        // analog comparator
#define ACD     7
#define ACBG    6
#define ACO     5
#define ACI     4
#define ACIE    3
#define ACIC    2
#define ACIS1   1
#define ACIS0   0

#define UBRRL   _SFR_IO8( 0x09)        // USART Baud Rate Register Low

#define UCSRB   _SFR_IO8( 0x0A)        // USART Control and Status Register B
#define RXCIE   7                      // RX Complete Interrupt Enable
#define TXCIE   6                      // TX Complete Interrupt Enable
#define UDRIE   5                      // USART Data Register Empty Interrupt Enable
#define RXEN    4                      // Receive Enable
#define TXEN    3                      // Transmitter Enable
#define UCSZ2   2                      // Character Size
#define RXB8    1                      // Receive Data Bit 8
#define TXB8    0                      // Transmit Data Bit 8

#define UCSRA   _SFR_IO8( 0x0B)        // USART Control and Status Register A
#define RXC     7                      // USART Receive Complete
#define TXC     6                      // USART Transmit Complete
#define UDRE    5                      // USART Data Register Empty
#define FE      4                      // Frame Error 
#define DOR     3                      // Data Overrun
#define PE      2                      // USART Parity Error
#define U2X     1                      // Double the USART Transmission Speed
#define MPCM    0                      // Multiprocessor Communication Mode

#define UDR     _SFR_IO8( 0x0C)        // USART I/O Data Register

#define SPCR    _SFR_IO8( 0x0D)        // SPI
#define SPIE    7
#define SPE     6
#define DORD    5
#define MSTR    4
#define CPOL    3
#define CPHA    2
#define SPR1    1
#define SPR0    0

#define SPSR    _SFR_IO8( 0x0E)
#define SPIF    7
#define WCOL    6
#define SPI2X   0

#define SPDR    _SFR_IO8( 0x0F)

#define PIND    _SFR_IO8( 0x10)        // Port D - Input Register

#define DDRD    _SFR_IO8( 0x11)        // Port D - Data Direction Register 

#define PORTD   _SFR_IO8( 0x12)        // Port D - Output Register

#define PINC    _SFR_IO8( 0x13)        // Port C - Input Register

#define DDRC    _SFR_IO8( 0x14)        // Port C - Data Direction Register

#define PORTC   _SFR_IO8( 0x15)        // Port C - Output Register

#define PINB    _SFR_IO8( 0x16)        // Port B - Input Register

#define DDRB    _SFR_IO8( 0x17)        // Port B - Data Direction Register

#define PORTB   _SFR_IO8( 0x18)        // Port B - Output Register

#define EECR    _SFR_IO8( 0x1C)        // EEPROM Control Register
#define EERIE   3                      // EEPROM Ready Interrupt Enabla Bit
#define EEMPE   2                      // EEPROM Master Program Enable Bit
#define EEPE    1                      // EEPROM Program Enable Bit
#define EERE    0                      // EEPROM Read Enable Bit

#define EEDR    _SFR_IO8( 0x1D)        // EEPROM Data Register

#define EEAR    _SFR_IO16( 0x1E)       // EEPROM Address Register
#define EEARL   _SFR_IO8( 0x1E)
#define EEARH   _SFR_IO8( 0x1F)

#define UCSRC   _SFR_IO8( 0x20)
#define URSEL   7
#define UMSEL   6
#define UPM1    5
#define UPM0    4
#define USBS    3
#define UCSZ1   2
#define UCSZ0   1
#define UCPOL   0

#define UBRRH   _SFR_IO8( 0x20)
#define URSEL   7

#define WDTCR   _SFR_IO8( 0x21)        // Watchdog Timer Control Register
// bits 7-5 reserved
#define WDCE    4
#define WDE     3
#define WDP2    2
#define WDP1    1
#define WDP0    0

#define ASSR    _SFR_IO8( 0x22)
// bits 7-4 reserved
#define AS2	3
#define TCN2UB	2
#define OCR2UB	1
#define TCR2UB	0

#define OCR2    _SFR_IO8( 0x23)        // Timer 2

#define TCNT2   _SFR_IO8( 0x24)

#define TCCR2   _SFR_IO8( 0x25)        // Timer/Counter 2 Control Register
#define FOC2	  7
#define WGM20	  6
#define COM21	  5
#define COM20	  4
#define WGM21	  3
#define CS22	  2
#define CS21	  1
#define CS20	  0

#define ICR1	 _SFR_IO16( 0x26)        // Input Capture Register 1
#define ICR1L	 _SFR_IO8( 0x26)
#define ICR1H  _SFR_IO8( 0x27)

#define OCR1B  _SFR_IO16( 0x28)        // Output Compare Register 1 B
#define OCR1BL _SFR_IO8( 0x28)
#define OCR1BH _SFR_IO8( 0x29)

#define OCR1A	 _SFR_IO16( 0x2A)        // Output Compare Register 1 A
#define OCR1AL _SFR_IO8( 0x2A)
#define OCR1AH _SFR_IO8( 0x2B)

#define TCNT1  _SFR_IO16( 0x2C)        // Timer/Counter 1
#define TCNT1L _SFR_IO8( 0x2C)
#define TCNT1H _SFR_IO8( 0x2D)

#define TCCR1B  _SFR_IO8( 0x2E)        // Timer/Counter 1 Control Register B
#define ICNC1   7
#define ICES1   6
// bit 5 reserved
#define WGM13   4
#define WGM12   3
#define CS12    2
#define CS11    1
#define CS10    0

#define TCCR1A  _SFR_IO8( 0x2F)       // Timer/Counter 1 Control Register A
#define COM1A1  7
#define COM1A0  6
#define COM1B1  5
#define COM1B0  4
#define FOC1A   3
#define FOC1B   2
#define WGM11   1
#define WGM10   0

#define SFIOR   _SFR_IO8( 0x30)        // Special Function IO Register
#define ACME    3
#define PUD     2                      // Pull-up Disable
#define PSR2    1
#define PSR10   0

#define OSCCAL  _SFR_IO8( 0x31)        // Oscillator Calibration Register

#define TCNT0   _SFR_IO8( 0x32)        // Timer/Counter Register 0 

#define TCCR0   _SFR_IO8( 0x33)        // Timer/Counter Control Register
#define CS02    2                      // Clock Set 2
#define CS01    1                      // Clock Set 1
#define CS00    0                      // Clock Set 0

#define MCUCSR  _SFR_IO8( 0x34)        // MCU Status Register
#define WDRF    3
#define BORF    2
#define EXTRF   1
#define PORF    0

#define MCUCR   _SFR_IO8( 0x35)        // MCU Control Register
#define SE      7
#define SM2     6
#define SM1     5
#define SM0     4
#define ISC11   3
#define ISC10   2
#define ISC01   1
#define ISC00   0

#define TWCR    _SFR_IO8( 0x36)
#define TWINT   7
#define TWEA    6
#define TWSTA   5
#define TWSTO   4
#define TWWC    3
#define TWEN    2
// bit 1 reserved (TWI_TST?)
#define TWIE    0

#define SPMCR   _SFR_IO8( 0x37)        // Store Program Memory Control Register
#define SPMIE   7
#define RWWSB   6
// bit 5 reserved
#define RWWSRE  4
#define BLBSET  3
#define PGWRT   2
#define PGERS   1
#define SPMEN   0

#define TIFR    _SFR_IO8( 0x38)        // Timer/Counter Interrupt Flag Register
#define OCF2    7
#define TOV2    6
#define ICF1    5
#define OCF1A   4
#define OCF1B   3
#define TOV1    2
// bit 1 reserved (OCF0?)
#define TOV0    0

#define TIMSK   _SFR_IO8( 0x39)        // Timer/Counter Interrupt Mask Register
#define OCIE2   7
#define TOIE2	  6
#define TICIE1	5
#define OCIE1A	4
#define OCIE1B	3
#define TOIE1   2
// bit 1 reserved (OCIE0?)
#define TOIE0   0

#define GIFR    _SFR_IO8( 0x3A)
#define INTF1   7
#define INTF0	  6

#define GICR	  _SFR_IO8( 0x3B)         // General Interrupt Control Register
#define INT1    7                       // External Interrupt Request 1 Enable
#define INT0    6                       // External Interrupt Request 0 Enable
#define IVSEL   1                       // External Interrupt Flag 1
#define IVCE    0                       // External Interrupt Flag 0


// 0x3C reserved (OCR0?)
// 0x3D..0x3E SP
// 0x3F SREG


// Interrupt vectors:
#define INT0_vect         _VECTOR( 1)  // External Interrupt Request 0
#define IT_INT0_ENABLE             BS(GICR,INT0)
#define IT_INT0_DISABLE            BC(GICR,INT0)

#define INT1_vect         _VECTOR( 2)  // External Interrupt Request 1
#define IT_INT1_ENABLE           BS(GICR,INT1)
#define IT_INT1_DISABLE          BC(GICR,INT1)

#define TIMER2_COMP_vect  _VECTOR( 3)  // Timer/Counter2 Compare Match
#define IT_TIMER2_COMP_ENABLE    BS(TIMSK,OCIE2)
#define IT_TIMER2_COMP_DISABLE   BC(TIMSK,OCIE2)

#define TIMER2_OVF_vect   _VECTOR( 4)  // Timer/Counter2 Overflow
#define IT_TIMER2_OVF_ENABLE     BS(TIMSK,TOIE2)
#define IT_TIMER2_OVF_DISABLE    BC(TIMSK,TOIE2)

#define TIMER1_CAPT_vect	_VECTOR( 5)  // Timer/Counter1 Capture Event
#define IT_TIMER1_CAPT_ENABLE    BS(TIMSK,TICIE1)
#define IT_TIMER1_CAPT_DISABLE   BC(TIMSK,TICIE1)

#define TIMER1_COMPA_vect	_VECTOR( 6)  // Timer/Counter1 Compare Match A
#define IT_TIMER1_COMPA_ENABLE   BS(TIMSK,OCIE1A)
#define IT_TIMER1_COMPA_DISABLE  BC(TIMSK,OCIE1A)

#define TIMER1_COMPB_vect	_VECTOR( 7)  // Timer/Counter1 Compare Match B
#define IT_TIMER1_COMPB_ENABLE   BS(TIMSK,OCIE1B)
#define IT_TIMER1_COMPB_DISABLE  BC(TIMSK,OCIE1B)

#define TIMER1_OVF_vect		_VECTOR( 8)  // Timer/Counter1 Overflow
#define IT_TIMER1_OVF_ENABLE     BS(TIMSK,TOIE1)
#define IT_TIMER1_OVF_DISABLE    BC(TIMSK,TOIE1)

#define TIMER0_OVF_vect		_VECTOR( 9)  // Timer/Counter0 Overflow
#define IT_TIMER0_OVF_ENABLE     BS(TIMSK,TOIE0)
#define IT_TIMER0_OVF_DISABLE    BC(TIMSK,TOIE0)

#define SPI_STC_vect			_VECTOR( 10) // Serial Transfer Complete
#define IT_SPI_STC_ENABLE        BS(SPCR,SPIE)
#define IT_SPI_STC_DISABLE       BC(SPCR,SPIE)

#define USART_RXC_vect		_VECTOR( 11) // USART, Rx Complete
#define IT_USART_RXC_ENABLE      BS(UCSRB,RXCIE)
#define IT_USART_RXC_DISABLE     BC(UCSRB,RXCIE)

#define USART_UDRE_vect		_VECTOR( 12) // USART Data Register Empty
#define IT_USART_UDRE_ENABLE     BS(UCSRB,UDRIE)
#define IT_USART_UDRE_DISABLE    BC(UCSRB,UDRIE)

#define USART_TXC_vect		_VECTOR( 13) // USART, Tx Complete
#define IT_USART_TXC_ENABLE      BS(UCSRB,TXCIE)
#define IT_USART_TXC_DISABLE     BC(UCSRB,TXCIE)

#define ADC_vect			    _VECTOR( 14) // ADC Conversion Complete
#define IT_ADC_ENABLE            BS(ADCSRA,ADIE)
#define IT_ADC_DISABLE           BC(ADCSRA,ADIE)

#define EE_RDY_vect			  _VECTOR( 15) // EEPROM Ready
#define IT_EE_RDY_ENABLE         BS(EECR,EERIE)
#define IT_EE_RDY_DISABLE        BC(EECR,EERIE)

#define ANA_COMP_vect			_VECTOR( 16) // Analog Comparator
#define IT_ANA_COMP_ENABLE       BS(ACSR,ACIE)
#define IT_ANA_COMP_DISABLE      BC(ACSR,ACIE)

#define TWI_vect			    _VECTOR( 17) // 2-wire Serial Interface
#define IT_TWI_ENABLE            BS(TWCR,TWIE)
#define IT_TWI_DISABLE           BC(TWCR,TWIE)

#define SPM_RDY_vect			_VECTOR( 18) // Store Program Memory Ready
#define IT_SPM_RDY_ENABLE        BS(SPMCR,SPMIE)
#define IT_SPM_RDY_DISABLE       BC(SPMCR,SPMIE)


#define ISR(vector) void vector (void) __attribute__ ((signal,used)); void vector (void)


/* Constants */
#define _VECTORS_SIZE 38
#define SPM_PAGESIZE    64
#define RAMEND       0x45F
#define XRAMEND     RAMEND
#define E2END        0x1FF
#define E2PAGESIZE       4
#define FLASHEND    0x1FFF


/* Fuses */
#define FUSE_MEMORY_SIZE 2

/* Low Fuse Byte */
#define FUSE_CKSEL0      (unsigned char)~_BV(0)
#define FUSE_CKSEL1      (unsigned char)~_BV(1)
#define FUSE_CKSEL2      (unsigned char)~_BV(2)
#define FUSE_CKSEL3      (unsigned char)~_BV(3)
#define FUSE_SUT0        (unsigned char)~_BV(4)
#define FUSE_SUT1        (unsigned char)~_BV(5)
#define FUSE_BODEN       (unsigned char)~_BV(6)
#define FUSE_BODLEVEL    (unsigned char)~_BV(7)
#define LFUSE_DEFAULT (FUSE_SUT0 & FUSE_CKSEL3 & FUSE_CKSEL2 & FUSE_CKSEL1)

/* High Fuse Byte */
#define FUSE_BOOTRST     (unsigned char)~_BV(0)
#define FUSE_BOOTSZ0     (unsigned char)~_BV(1)
#define FUSE_BOOTSZ1     (unsigned char)~_BV(2)
#define FUSE_EESAVE      (unsigned char)~_BV(3)
#define FUSE_CKOPT       (unsigned char)~_BV(4)
#define FUSE_SPIEN       (unsigned char)~_BV(5)
#define FUSE_WDTON       (unsigned char)~_BV(6)
#define FUSE_RSTDISBL    (unsigned char)~_BV(7)
#define HFUSE_DEFAULT (FUSE_SPIEN & FUSE_BOOTSZ1 & FUSE_BOOTSZ0)


/* Lock Bits */
#define __LOCK_BITS_EXIST
#define __BOOT_LOCK_BITS_0_EXIST
#define __BOOT_LOCK_BITS_1_EXIST 


/* Signature */
#define SIGNATURE_0 0x1E
#define SIGNATURE_1 0x93
#define SIGNATURE_2 0x07




// KT macro-k

#define U8                   unsigned char
#define U16                  unsigned int

#define BS(sfr,bit)          sfr|= ( 1 << ( bit))               // BIT-SET - egy regiszter adott bitjenek 1-re allitas
#define BC(sfr,bit)          sfr&= ~( 1 << ( bit))              // BIT-CLEAR - egy regiszter adott bitjenek 0-ra allitas
#define BTS(sfr,bit)         bit_is_set( sfr, bit)              // BIT-TEST-SET - egy regiszter egy bitjenek vizsgalata 1-re
#define BTC(sfr,bit)         bit_is_clear( sfr, bit)            // BIT-TEST-CLEAR - egy regiszter egy bitjenek vizsgalata 0-ra

#define IT_ENABLE            __asm__ __volatile__ ("sei" ::)    // megszakitas engedelyezese
#define IT_DISABLE           __asm__ __volatile__ ("cli" ::)    // megszakitas tiltasa
#define NOP                  __asm__ __volatile__ ("nop" ::)    // egy ures utasitas beszurasa

#define PIN_B0_OUT           BS( DDRB, 0)          
#define PIN_B1_OUT           BS( DDRB, 1)
#define PIN_B2_OUT           BS( DDRB, 2)
#define PIN_B3_OUT           BS( DDRB, 3)
#define PIN_B4_OUT           BS( DDRB, 4)
#define PIN_B5_OUT           BS( DDRB, 5)
#define PIN_B6_OUT           BS( DDRB, 6)
#define PIN_B7_OUT           BS( DDRB, 7)

#define PIN_C0_OUT           BS( DDRC, 0)
#define PIN_C1_OUT           BS( DDRC, 1)
#define PIN_C2_OUT           BS( DDRC, 2)
#define PIN_C3_OUT           BS( DDRC, 3)
#define PIN_C4_OUT           BS( DDRC, 4)
#define PIN_C5_OUT           BS( DDRC, 5)
#define PIN_C6_OUT           BS( DDRC, 6)

#define PIN_D0_OUT           BS( DDRD, 0)
#define PIN_D1_OUT           BS( DDRD, 1)
#define PIN_D2_OUT           BS( DDRD, 2)
#define PIN_D3_OUT           BS( DDRD, 3)
#define PIN_D4_OUT           BS( DDRD, 4)
#define PIN_D5_OUT           BS( DDRD, 5)
#define PIN_D6_OUT           BS( DDRD, 6)
#define PIN_D7_OUT           BS( DDRD, 7)


// USART-hoz

// UCSRA beallitasai
#define DOUBLE_SPEED         0B00000010          // ketszeres sebesseg

// UCSRB beallitasai
#define IT_RX_ENABLE         0B10000000          // vetel kesz megszakitas engedelyezese
#define IT_TX_ENABLE         0B01000000          // adas kesz megszakitas engedelyezese
#define IT_UDR_EMPTY_ENABLE  0B00100000          // adat buffer ures megszakitas engedelyezese
#define RX_ENABLE            0B00010000          // vetel engedelyezese
#define TX_ENABLE            0B00001000          // adas engedelyezese

// UCSRC beallitasai
#define SET_UCSRC            0B10000000          // UCSRC kivalasztasa
#define SET_UBRRH            0                   // UBRRB kivalasztasa
#define ASYNCRON             0                   // asszinkron mukodes
#define SYNCRON              0B01000000          // szinkron mukodes
#define PARITY_NO            0                   // nincs paritas bit
#define PARITY_EVEN          0B00100000          // paros paritas
#define PARITY_ODD           0B00110000          // paratlan paritas
#define STOPBIT_1            0                   // 1 stopbit
#define STOPBIT_2            0B00001000          // 2 stopbit
#define BIT_5                0                   // 5 bit-es karakterhossz
#define BIT_6                0B00000010          // 6 bit-es karakterhossz
#define BIT_7                0B00000100          // 7 bit-es karakterhossz
#define BIT_8                0B00000110          // 8 bit-es karakterhossz

#define KTUBRR               F_CPU/ 16UL/ BAUDRATE - 1

#define USART_SET_9600_8N1   UBRRL= F_CPU/ 16UL/ 9600- 1;  \
                             UBRRH= ( F_CPU/ 16UL/ 9600- 1)>> 8;  \
                             UCSRC= SET_UCSRC+ ASYNCRON+ BIT_8+ PARITY_NO+ STOPBIT_1;  \
                             UCSRB= RX_ENABLE+ TX_ENABLE+ IT_RX_ENABLE

#define USART_SET_19200_8N1  UBRRL= F_CPU/ 16UL/ 19200- 1;  \
                             UBRRH= ( F_CPU/ 16UL/ 19200- 1)>> 8;  \
                             UCSRC= SET_UCSRC+ ASYNCRON+ BIT_8+ PARITY_NO+ STOPBIT_1;  \
                             UCSRB= RX_ENABLE+ TX_ENABLE+ IT_RX_ENABLE

#define USART_SET_115200_8N1 UBRRL=F_CPU/ 16UL/ 115200- 1;  \
                             UBRRH=( F_CPU/ 16UL/ 115200- 1)>> 8;  \
                             UCSRC=SET_UCSRC+ ASYNCRON+ BIT_8+ PARITY_NO+ STOPBIT_1;  \
                             UCSRB=RX_ENABLE+ TX_ENABLE+ IT_RX_ENABLE


// MCUCR beallitasai
#define INT0_0               0                   // IT 0 szintnel
#define INT0_1               0B00000001          // IT 1 szintnel
#define INT0_LEFUTO          0B00000010          // IT 1->0 elre
#define INT0_FELFUTO         0B00000011          // IT 0->1 elre
#define INT1_0               0                   // IT 0 szintnel
#define INT1_1               0B00000100          // IT 1 szintnel
#define INT1_LEFUTO          0B00001000          // IT 1->0 elre
#define INT1_FELFUTO         0B00001100          // IT 0->1 elre


// ADC-hez

// ADMUX beallitasai
#define VREF_AREF            0                   // kulso referencia feszultseg
#define VREF_AVCC            0B01000000          // az analog tapfesz a referencia
#define VREF_INT_2_56V       0B11000000          // belso 2,56V referencia
#define ADC_8BIT             0B00100000          // 8 bit felbontas ADCH-bol
#define ADC_10BIT            0                   // 10 bit felbontas, ADCH+ ADCL
#define IN_ADC0              0                   // bemenet valasztas
#define IN_ADC1              0B00000001
#define IN_ADC2              0B00000010
#define IN_ADC3              0B00000011
#define IN_ADC4              0B00000100
#define IN_ADC5              0B00000101
#define IN_ADC6              0B00000110
#define IN_ADC7              0B00000111
#define IN_VBG               0B00001110
#define IN_GND               0B00001111

// ADCSRA  beallitasai
#define ADC_ENABLE           0B10000000          // ADC engedelyezes
#define ADC_START            0B01000000          // AD meres inditasa
#define ADC_FREE_RUN         0B00100000          // AD folyamatos meres
#define ADC_IT_FLAG          0B00010000
#define ADC_IT_ENABLE        0B00001000          // ADC megszakitas engedelyezese
#define ADC_SC_2             0B00000001          // F_CPU/ osztas - 50-200kHZ beallitando
#define ADC_SC_4             0B00000010
#define ADC_SC_8             0B00000011
#define ADC_SC_16            0B00000100
#define ADC_SC_32            0B00000101
#define ADC_SC_64            0B00000110
#define ADC_SC_128           0B00000111
#define START_ADC            ADCSRA|=ADC_START   // AD merest indito macro 





// T0 8 bit-es szamlalo, a legegyszerubb. 
// Elsosorban kulso impulzusok szamlalasara es idozitesre alkalmas.
// OVF IT-t akkor general, amikor TCNT0 255-rol 0-ra ugrik.
// 0..255-0 szamlal, erteke kozben atirhato.
// NORMAL modban belso orajelrol jar 
// Regiszterek - TCNT0, TCCR0, TIMSK, TIFR 
// Az elooszto erteket kell legeloszor megadni, ettol fuggenek az uzemmodok.
//
#ifndef T0_DIVIDER
   #define T0_DIVIDER       1                   // 1-8-64-256-1024
#endif
#define T0_STOP               TCCR0=0
#if T0_DIVIDER == 1
   #define T0_NORMAL          TCCR0=0B00000001
#endif
#if T0_DIVIDER == 8
   #define T0_NORMAL          TCCR0=0B00000010
#endif
#if T0_DIVIDER == 64
   #define T0_NORMAL          TCCR0=0B00000011
#endif
#if T0_DIVIDER == 256
   #define T0_NORMAL          TCCR0=0B00000100
#endif
#if T0_DIVIDER == 1024
   #define T0_NORMAL          TCCR0=0B00000101
#endif
#define T0_COUNT_EXTERN_FALL  TCCR0=0B00000110
#define T0_COUNT_EXTERN_RISE  TCCR0=0B00000111
#define T0_IT_OVF_ENABLE      BS(TIMSK,TOIE0)
#define T0_IT_OVF_DISABLE     BC(TIMSK,TOIE0)
#define T0_T(usec)            TCNT0=255-(usec/((T0_DIVIDER*1000000)/F_CPU))

 
// T1 16 bit-es szamlalo, szamos uzemoddal (nem mind lett definialva).
// Regiszterek - TCNT1, TCCR1A, TCCR1B, OCR1A, OCR1B, ICR1, TIMSK, ACSR, TIFR, SFIOR  
// Az elooszto erteket kell legeloszor megadni, ettol fuggenek az uzemmodok.
//
#ifndef T1_DIVIDER
   #define T1_DIVIDER           1                 // 1-8-64-256-1024
#endif
#define OC1A_OUTPUT_ENABLE      BS(DDRB,1)
#define OC1B_OUTPUT_ENABLE      BS(DDRB,2)
// Uzemmodok
#define T1_STOP                 TCCR1B=0
#if T1_DIVIDER == 1
   #define T1_NORMAL            TCCR1A=0B00000000; TCCR1B=0B00000001
   #define T1_CTC               TCCR1A=0B00000000; TCCR1B=0B00001001
   #define T1_PWM8              TCCR1A=0B10000001; TCCR1B=0B00001001; OC1A_OUTPUT_ENABLE
   #define T1_PWM10             TCCR1A=0B10000011; TCCR1B=0B00001001; OC1A_OUTPUT_ENABLE
   #define T1_CAPTURE           TCCR1A=0B00000000; TCCR1B=0B00000001
   #define T1_SWAVE             TCCR1A=0B01000000; TCCR1B=0B00011001; OC1A_OUTPUT_ENABLE
#endif
#if T1_DIVIDER == 8
   #define T1_NORMAL            TCCR1A=0B00000000; TCCR1B=0B00000010
   #define T1_CTC               TCCR1A=0B00000000; TCCR1B=0B00001010
   #define T1_PWM8              TCCR1A=0B10000001; TCCR1B=0B00001010; OC1A_OUTPUT_ENABLE
   #define T1_PWM10             TCCR1A=0B10000011; TCCR1B=0B00001010; OC1A_OUTPUT_ENABLE
   #define T1_CAPTURE           TCCR1A=0B00000000; TCCR1B=0B00000010
   #define T1_SWAVE             TCCR1A=0B01000000; TCCR1B=0B00011010; OC1A_OUTPUT_ENABLE
#endif
#if T1_DIVIDER == 64
   #define T1_NORMAL            TCCR1A=0B00000000; TCCR1B=0B00000011
   #define T1_CTC               TCCR1A=0B00000000; TCCR1B=0B00001011
   #define T1_PWM8              TCCR1A=0B10000001; TCCR1B=0B00001011; OC1A_OUTPUT_ENABLE
   #define T1_PWM10             TCCR1A=0B10000011; TCCR1B=0B00001011; OC1A_OUTPUT_ENABLE
   #define T1_CAPTURE           TCCR1A=0B00000000; TCCR1B=0B00000011
   #define T1_SWAVE             TCCR1A=0B01000000; TCCR1B=0B00011011; OC1A_OUTPUT_ENABLE
#endif
#if T1_DIVIDER == 256
   #define T1_NORMAL            TCCR1A=0B00000000; TCCR1B=0B00000100
   #define T1_CTC               TCCR1A=0B00000000; TCCR1B=0B00001100
   #define T1_PWM8              TCCR1A=0B10000001; TCCR1B=0B00001100; OC1A_OUTPUT_ENABLE
   #define T1_PWM10             TCCR1A=0B10000011; TCCR1B=0B00001100; OC1A_OUTPUT_ENABLE
   #define T1_CAPTURE           TCCR1A=0B00000000; TCCR1B=0B00000100
   #define T1_SWAVE             TCCR1A=0B01000000; TCCR1B=0B00011101; OC1A_OUTPUT_ENABLE
#endif
#if T1_DIVIDER == 1024
   #define T1_NORMAL            TCCR1A=0B00000000; TCCR1B=0B00000101
   #define T1_CTC               TCCR1A=0B00000000; TCCR1B=0B00001101
   #define T1_PWM8              TCCR1A=0B10000001; TCCR1B=0B00001101; OC1A_OUTPUT_ENABLE
   #define T1_PWM10             TCCR1A=0B10000011; TCCR1B=0B00001101; OC1A_OUTPUT_ENABLE
   #define T1_CAPTURE           TCCR1A=0B00000000; TCCR1B=0B00000101
   #define T1_SWAVE             TCCR1A=0B01000000; TCCR1B=0B00011101; OC1A_OUTPUT_ENABLE
#endif
#define T1_COUNT_EXTERN_FALL    TCCR1A=0B00000000; TCCR1B=0B00000110
#define T1_COUNT_EXTERN_RISE    TCCR1A=0B00000000; TCCR1B=0B00000111
// Parancsok
#define T0_T1_DIVIDER_RESET     BS(SFIOR,PSR10)            // kozos T0-lal
#define T1_CAPTURE_RISE         BS(TCCR1B,ICES1)
#define T1_CAPTURE_FALL         BC(TCCR1B,ICES1)
#define T1_CAPTURE_ACOMP        BS(ACSR,ACIC)
#define T1_CAPTURE_ICP0         BC(ACSR,ACIC)
#define T1_IT_COMPA_ENABLE      BS(TIMSK,OCIE1A)
#define T1_IT_COMPA_DISABLE     BC(TIMSK,OCIE1A)
#define T1_IT_COMPB_ENABLE      BS(TIMSK,OCIE1B)
#define T1_IT_COMPB_DISABLE     BC(TIMSK,OCIE1B)
#define T1_IT_OVF_ENABLE        BS(TIMSK,TOIE1)
#define T1_IT_OVF_DISABLE       BC(TIMSK,TOIE1)
#define T1_IT_CAPTURE_ENABLE    BS(TIMSK,TICIE1)
#define T1_IT_CAPTURE_DISABLE   BC(TIMSK,TICIE1)
#define T1_SWAVE_F(hz)          ICR1=F_CPU/T1_DIVIDER/2/hz-1  
#define T1_PWM8_RATE(percent)   OCR1A=F_CPU/T1_DIVIDER/(percent*255/100)  
#define T1_PWM10_RATE(percent)  OCR1A=F_CPU/T1_DIVIDER/(percent*1023/100)  
#define T1_CTC_F(hz)            OCR1A=F_CPU/T1_DIVIDER/hz-1  
#define T1_CTC_T(usec)          OCR1A=usec/((T1_DIVIDER*1000000)/F_CPU-1)  
// T1_PWM8_F= F_CPU/T1_PRESCALER/256  
// T1_PWM10_F= F_CPU/T1_PRESCALER/1024  




// T2 8 bit-es szamlalo, tobbfele uzemoddal, de csak belso orajelrol jar.
// OVF IT-t akkor general, amikor TCNT0 255-rol 0-ra ugrik.
// COMP IT-t akkor general, amikor TCNT0 erteke OCR2 ertekevel megegyezik.
// NORMAL modban TCNT2 0..255-0 szamlal, erteke kozben atirhato.
// CTC modban TCNT2 0..OCR2-0 szamlal. 
// PWM modban TCNT2 0..255-0 szamlal, OCR2-ig OC2=0 (kimenet).  
// SWAVE mod olyan mint CTC, de OC2 (kimenet) allapota minden esetben megvaltozik, negyszog jelet general.  
// Tobbfele kombinacio is beallithato, bitos van ertelmuk is...
// Regiszterek - TCNT2, TCCR2, OCR2, TIMSK, SFIOR, TIFR  
// Az elooszto erteket kell legeloszor megadni, ettol fuggenek az uzemmodok.
//
#ifndef T2_DIVIDER
   #define T2_DIVIDER         1                  // 1-8-32-64-128-256-1024
#endif
#define OC2_OUTPUT_ENABLE     BS(DDRB,3)
// Uzemmodok
#define T2_STOP               TCCR2=0
#if T2_DIVIDER == 1
   #define T2_NORMAL          TCCR2=0B00000001
   #define T2_PWM             TCCR2=0B01101001; OC2_OUTPUT_ENABLE
   #define T2_CTC             TCCR2=0B00001001
   #define T2_SWAVE           TCCR2=0B00011001; OC2_OUTPUT_ENABLE
#endif
#if T2_DIVIDER == 8
   #define T2_NORMAL          TCCR2=0B00000010
   #define T2_PWM             TCCR2=0B01101010; OC2_OUTPUT_ENABLE
   #define T2_CTC             TCCR2=0B00001010
   #define T2_SWAVE           TCCR2=0B00011010; OC2_OUTPUT_ENABLE
#endif
#if T2_DIVIDER == 32
   #define T2_NORMAL          TCCR2=0B00000011
   #define T2_PWM             TCCR2=0B01101011; OC2_OUTPUT_ENABLE
   #define T2_CTC             TCCR2=0B00001011
   #define T2_SWAVE           TCCR2=0B00011011; OC2_OUTPUT_ENABLE
#endif
#if T2_DIVIDER == 64
   #define T2_NORMAL          TCCR2=0B00000100
   #define T2_PWM             TCCR2=0B01101100; OC2_OUTPUT_ENABLE
   #define T2_CTC             TCCR2=0B00001100
   #define T2_SWAVE           TCCR2=0B00011100; OC2_OUTPUT_ENABLE
#endif
#if T2_DIVIDER == 128
   #define T2_NORMAL          TCCR2=0B00000101
   #define T2_PWM             TCCR2=0B01101101; OC2_OUTPUT_ENABLE
   #define T2_CTC             TCCR2=0B00001101
   #define T2_SWAVE           TCCR2=0B00011101; OC2_OUTPUT_ENABLE
#endif
#if T2_DIVIDER == 256
   #define T2_NORMAL          TCCR2=0B00000110
   #define T2_PWM             TCCR2=0B01101110; OC2_OUTPUT_ENABLE
   #define T2_CTC             TCCR2=0B00001110
   #define T2_SWAVE           TCCR2=0B00011110; OC2_OUTPUT_ENABLE
#endif
#if T2_DIVIDER == 1024
   #define T2_NORMAL          TCCR2=0B00000111
   #define T2_PWM             TCCR2=0B01101111; OC2_OUTPUT_ENABLE
   #define T2_CTC             TCCR2=0B00001111
   #define T2_SWAVE           TCCR2=0B00011111; OC2_OUTPUT_ENABLE
#endif
// Parancsok
#define T2_DIVIDER_RESET      BS(SFIOR,PSR2)
#define T2_IT_OVF_ENABLE      BS(TIMSK,TOIE2)
#define T2_IT_OVF_DISABLE     BC(TIMSK,TOIE2)
#define T2_IT_COMP_ENABLE     BS(TIMSK,OCIE2)
#define T2_IT_COMP_DISABLE    BC(TIMSK,OCIE2)
#define T2_SWAVE_F(hz)        OCR2=F_CPU/T2_DIVIDER/2/hz-1  
#define T2_PWM_RATE(percent)  OCR2=F_CPU/T2_DIVIDER/(percent*255/100)  
#define T2_CTC_F(hz)          OCR2=F_CPU/T2_DIVIDER/hz-1  
#define T2_CTC_T(usec)        OCR2=usec/((T2_DIVIDER*1000000)/F_CPU-1)  
// T2_PWM_F= F_CPU/T2_PRESCALER/256  
